"use strict";
'use client';
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buttonResetStyles = void 0;
exports.default = Button;
const react_1 = __importDefault(require("react"));
const react_aria_components_1 = require("react-aria-components");
const core_1 = require("@kuma-ui/core");
const cx_1 = __importDefault(require("../utilities/cx"));
const skeleton_1 = require("./skeleton");
const loading_1 = __importDefault(require("./loading"));
const font_1 = require("./font");
const box_1 = __importDefault(require("./box"));
const moving_gradients_1 = __importDefault(require("./moving-gradients"));
const moving_stars_1 = __importDefault(require("./moving-stars"));
function ButtonIcon({ iconLeft, iconRight, }) {
    if (!iconLeft && !iconRight) {
        return null;
    }
    return (react_1.default.createElement("div", { className: (0, cx_1.default)((0, core_1.css) `
					color: var(--foreground-color);
					width: 0.9em;
					height: 0.9em;
					position: relative;
					display: inline-flex;
					margin: auto;
				`, Boolean(iconLeft) &&
            (0, core_1.css) `
						left: -0.25em;
					`, Boolean(iconRight) &&
            (0, core_1.css) `
						right: -0.25em;
					`) }, iconLeft !== null && iconLeft !== void 0 ? iconLeft : iconRight));
}
exports.buttonResetStyles = (0, cx_1.default)(font_1.fontFamilyStyles, (0, core_1.css) `
		appearance: none;
		background: transparent;
		text-align: left;
		font-size: 16px;
		line-height: 1em;
		color: var(--foreground-color);
		letter-spacing: normal;
		text-indent: 0;
		word-spacing: normal;
		border: 0;
		margin: 0;
		padding: 0;
		cursor: pointer;
		vertical-align: top;

		:where([dir='rtl']) & {
			text-align: right;
		}
	`);
const primaryStyles = (0, core_1.css) `
	--border-radius: 20px;
	background: var(--primary-color);
	padding: var(--half-indent) var(--indent);
	border-radius: var(--border-radius);

	--text-shadow-color: rgba(0, 0, 0, 0.2);
	--highlight-shadow-color: rgba(255, 255, 255, 0.075);
	--outline-shadow-color: rgba(0, 0, 0, 0.3);
	--foreground-color: #fff;
	text-shadow: 0 1px 0 var(--text-shadow-color);

	:where([data-theme='dark']) & {
		--highlight-shadow-color: rgba(0, 0, 0, 0.075);
		--outline-shadow-color: rgba(255, 255, 255, 0.125);
	}

	&:focus {
		outline: none;
	}

	&:focus-visible {
		outline: normal;
	}

	&:after {
		--offset: 1px;
		content: '';
		background: rgba(255, 255, 255, 0);
		position: absolute;
		top: var(--offset);
		right: var(--offset);
		bottom: var(--offset);
		left: var(--offset);
		border-radius: var(--border-radius);
		transition: background-color 0.37s;
		box-shadow:
			0 0 0 1px var(--outline-shadow-color),
			inset 0 0 0 1px var(--highlight-shadow-color);
		pointer-events: none;
		z-index: 1;
	}

	@media (hover: hover) {
		&:hover {
			transition-duration: 0.1s;
		}

		&:hover:after {
			background-color: rgba(255, 255, 255, 0.06);
			transition-duration: 0.1s;

			[data-theme='dark'] & {
				background-color: rgba(255, 255, 255, 0.04);
			}
		}
	}
`;
const secondaryStyles = (0, cx_1.default)(primaryStyles, (0, core_1.css) `
		background-color: var(--secondary-color);
	`);
const primaryWrapperStyles = (0, core_1.css) `
	--border-radius: 20px;
`;
const secondaryWrapperStyles = primaryWrapperStyles;
const buttonOverlayStyles = (0, core_1.css) `
	--offset: 0px;
	border-radius: var(--border-radius);
	position: absolute;
	top: var(--offset);
	left: var(--offset);
	right: var(--offset);
	bottom: var(--offset);
	pointer-events: none;
`;
function Button(_a) {
    var { className, children: inputChildren, variant = 'primary', iconLeft, iconRight, isLoading, isSkeleton, hasMovingGradients = false, movingGradientsCount = 4, hasMovingStars = false, movingStarsCount = 10 } = _a, properties = __rest(_a, ["className", "children", "variant", "iconLeft", "iconRight", "isLoading", "isSkeleton", "hasMovingGradients", "movingGradientsCount", "hasMovingStars", "movingStarsCount"]);
    const classNames = (0, cx_1.default)(exports.buttonResetStyles, (0, core_1.css) `
			font-weight: calc(500 + var(--font-weight-modifier));
			position: relative;

			&:not([disabled]):active {
				transform: scale(0.98);
				transition-duration: 0.1s;
			}

			@media (hover: hover) {
				&:hover {
					text-decoration: underline;
				}
			}
		`, variant === 'primary' && primaryStyles, variant === 'secondary' && secondaryStyles, isSkeleton &&
        (0, core_1.css) `
				visibility: hidden;
			`);
    const showMovingGradients = hasMovingGradients && movingGradientsCount > 0;
    const showMovingStars = hasMovingStars && movingStarsCount > 0;
    const children = (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(box_1.default, { className: (0, core_1.css) `
					position: relative;
					z-index: 1;
					gap: 0.15em;
				` },
            react_1.default.createElement(ButtonIcon, { iconLeft: iconLeft }),
            inputChildren,
            react_1.default.createElement(ButtonIcon, { iconRight: iconRight })),
        (showMovingGradients !== null && showMovingGradients !== void 0 ? showMovingGradients : showMovingStars) ? (react_1.default.createElement("div", { className: (0, cx_1.default)(buttonOverlayStyles, (0, core_1.css) `
							overflow: hidden;
						`) },
            showMovingGradients ? (react_1.default.createElement(moving_gradients_1.default, { maxGradientsCount: movingGradientsCount, opacity: 1 })) : null,
            showMovingStars ? (react_1.default.createElement(moving_stars_1.default, { maxStarsCount: movingStarsCount, className: (0, core_1.css) `
								color: white;
							` })) : null)) : null));
    const component = 'href' in properties ? (react_1.default.createElement(react_aria_components_1.Link, Object.assign({}, properties, { className: (0, cx_1.default)(classNames, (0, core_1.css) `
						text-decoration: none;
						display: inline-flex;
					`) }), children)) : (react_1.default.createElement(react_aria_components_1.Button, Object.assign({}, properties, { isDisabled: Boolean(isLoading) || Boolean(isSkeleton) || properties.isDisabled, className: classNames }), children));
    return (react_1.default.createElement("div", { className: (0, cx_1.default)((0, core_1.css) `
					position: relative;
					display: inline-flex;
				`, variant === 'primary' && primaryWrapperStyles, variant === 'secondary' && secondaryWrapperStyles) },
        component,
        react_1.default.createElement("div", { className: (0, cx_1.default)(buttonOverlayStyles, (0, core_1.css) `
						--offset: 1px;
						--blur: 25px;
						background-color: rgba(0, 0, 0, 0.2);
						backdrop-filter: blur(var(--blur));
						-webkit-backdrop-filter: blur(var(--blur));
						box-shadow: inset 0 0 0 1px rgba(0, 0, 0, 0.15);
						box-sizing: border-box;
						text-align: center;
						opacity: 0;
						transition: opacity 0.37s;
						z-index: 1;
					`, isLoading &&
                (0, core_1.css) `
							opacity: 1;
							transition-duration: 0.2s;
						`) }, isLoading ? (react_1.default.createElement("div", { className: (0, core_1.css) `
							display: inline-flex;
							position: absolute;
							top: round(50%, 1px);
							left: round(50%, 1px);
							transform: translate(-50%, -50%);
						` },
            react_1.default.createElement(loading_1.default, { size: variant === 'stripped' ? 10 : undefined, strokeWidth: variant === 'stripped' ? 1.5 : undefined, className: (0, core_1.css) `
								--color: #fff;

								svg {
									filter: drop-shadow(0 0 5px rgba(0, 0, 0, 0.3))
										drop-shadow(0 0 1px rgba(0, 0, 0, 0.4));
								}
							` }))) : null),
        isSkeleton ? react_1.default.createElement(skeleton_1.SkeletonInstance, { className: buttonOverlayStyles }) : null));
}
