"use strict";
'use client';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = MovingGradients;
const react_1 = __importStar(require("react"));
const core_1 = require("@kuma-ui/core");
const cx_1 = __importDefault(require("../utilities/cx"));
const themes = {
    main: [
        [73, 51, 205],
        [97, 24, 190],
        [116, 42, 236],
        [189, 0, 255],
        [53, 2, 195],
        [69, 28, 157],
        [49, 37, 204],
    ],
    noir: [
        [20, 20, 20],
        [30, 30, 30],
        [40, 40, 40],
        [50, 50, 50],
    ],
};
let colors = themes.noir;
if (typeof window !== 'undefined' && 'MutationObserver' in window) {
    const observer = new MutationObserver(() => {
        const { variant } = document.documentElement.dataset;
        colors =
            variant && variant in themes
                ? themes[variant]
                : themes.main;
    });
    observer.observe(document.documentElement, { attributes: true });
}
const multiplier = 1.25;
const colorsLength = colors.length;
function getGradient(canvasWidth, canvasHeight, speed = 1) {
    const randomness = Math.random();
    const color = colors[Math.floor(colorsLength * randomness)];
    const extendedCanvasWidth = canvasWidth * multiplier;
    const quarterExtendedCanvasWidth = canvasWidth * 0.25;
    const extendedCanvasHeight = canvasHeight * multiplier;
    const quarterExtendedCanvasHeight = canvasHeight * 0.25;
    const startPositionX = extendedCanvasWidth * Math.random() - quarterExtendedCanvasWidth;
    const startPositionY = extendedCanvasHeight * Math.random() - quarterExtendedCanvasHeight;
    const maximumDimension = Math.max(extendedCanvasWidth, extendedCanvasHeight);
    return {
        color,
        size: maximumDimension * randomness + maximumDimension / 2,
        opacity: 0,
        speed: Math.max(0.1, (randomness / 2) * speed),
        startPosition: [startPositionX, startPositionY],
        currentPosition: [startPositionX, startPositionY],
        targetPosition: [
            extendedCanvasWidth * Math.random() - quarterExtendedCanvasWidth,
            extendedCanvasHeight * Math.random() - quarterExtendedCanvasHeight,
        ],
    };
}
function MovingGradients({ className, maxGradientsCount = 10, speed = 1, opacity = 0.2, }) {
    const canvas = (0, react_1.useRef)(null);
    const [context, setContext] = (0, react_1.useState)();
    const [width, setWidth] = (0, react_1.useState)(0);
    const [height, setHeight] = (0, react_1.useState)(0);
    (0, react_1.useEffect)(() => {
        const gradients = Array.from({ length: maxGradientsCount }).map(() => getGradient(width, height, speed));
        let currentGradientsCount = maxGradientsCount;
        let isCancelled = false;
        let frame;
        function loop() {
            if (isCancelled) {
                return;
            }
            if (!context) {
                frame = requestAnimationFrame(loop);
                return;
            }
            context.clearRect(0, 0, width, height);
            for (let index = gradients.length; index; index -= 1) {
                const gradient = gradients[index - 1];
                const [startX, startY] = gradient.startPosition;
                const [currentX, currentY] = gradient.currentPosition;
                const [targetX, targetY] = gradient.targetPosition;
                let isFulfilled = true;
                if (startX > targetX) {
                    if (currentX > targetX) {
                        isFulfilled = false;
                    }
                }
                else if (currentX < targetX) {
                    isFulfilled = false;
                }
                if (startY > targetY) {
                    if (currentY > targetY) {
                        isFulfilled = false;
                    }
                }
                else if (currentY < targetY) {
                    isFulfilled = false;
                }
                if (isFulfilled) {
                    gradients.splice(index - 1, 1);
                    currentGradientsCount -= 1;
                    continue;
                }
                const middleX = (startX + targetX) / 2 - startX;
                const distanceX = currentX - startX;
                const middleY = (startY + targetY) / 2 - startY;
                const distanceY = currentY - startY;
                const opacity = (distanceX / middleX + distanceY / middleY) / 2;
                gradient.opacity = opacity > 1 ? 2 - opacity : opacity;
                const radius = gradient.size / 2;
                const fill = context.createRadialGradient(currentX, currentY, radius, currentX, currentY, 0);
                fill.addColorStop(0, `rgba(${gradient.color.join(',')},0)`);
                fill.addColorStop(1, `rgba(${gradient.color.join(',')},${gradient.opacity})`);
                context.fillStyle = fill;
                context.fillRect(currentX - radius, currentY - radius, gradient.size, gradient.size);
                if (currentX !== targetX) {
                    const varianceX = Math.abs(startX - targetX);
                    const varianceY = Math.abs(startY - targetY);
                    const multiplier = Math.min(1, varianceX / varianceY);
                    if (currentX < targetX) {
                        gradient.currentPosition[0] += gradient.speed * multiplier;
                    }
                    else {
                        gradient.currentPosition[0] -= gradient.speed * multiplier;
                    }
                }
                if (currentY !== targetY) {
                    const varianceX = Math.abs(startX - targetX);
                    const varianceY = Math.abs(startY - targetY);
                    const multiplier = Math.min(1, varianceY / varianceX);
                    if (currentY < targetY) {
                        gradient.currentPosition[1] += gradient.speed * multiplier;
                    }
                    else {
                        gradient.currentPosition[1] -= gradient.speed * multiplier;
                    }
                }
            }
            frame = requestAnimationFrame(loop);
        }
        frame = requestAnimationFrame(loop);
        const interval = setInterval(() => {
            let newGradientsCount = 10 - currentGradientsCount;
            if (newGradientsCount > 0) {
                while (newGradientsCount) {
                    gradients.push(getGradient(width, height));
                    newGradientsCount -= 1;
                    currentGradientsCount += 1;
                }
            }
        }, 1500);
        return () => {
            isCancelled = true;
            if (frame) {
                cancelAnimationFrame(frame);
            }
            clearInterval(interval);
        };
    }, [context, width, height, maxGradientsCount, speed]);
    (0, react_1.useEffect)(() => {
        if (canvas.current) {
            const context = canvas.current.getContext('2d');
            if (context) {
                setContext(context);
            }
            setWidth(canvas.current.offsetWidth);
            setHeight(canvas.current.offsetHeight);
        }
    }, [canvas]);
    return (react_1.default.createElement("canvas", { ref: canvas, width, height, style: { '--opacity': opacity }, className: (0, cx_1.default)((0, core_1.css) `
					width: 100%;
					height: 100%;
					pointer-events: none;
					position: absolute;
					top: 0;
					right: 0;
					bottom: 0;
					left: 0;
					opacity: var(--opacity);

					:where([data-theme='dark']) & {
						opacity: calc(var(--opacity) / 2);
					}
				`, className) }));
}
